﻿import { world,system, ItemStack } from "@minecraft/server";
import "blocks.js";



const workstations = {
    "minecraft:smithing_table": "toolsmith",
    "minecraft:grindstone": "weaponsmith",
    "minecraft:cartography_table": "cartographer",
    "minecraft:barrel": "fisherman",
    "minecraft:blast_furnace": "armorer",
    "minecraft:brewing_stand": "cleric",
    "minecraft:composter": "farmer",
    "minecraft:lectern": "librarian",
    "minecraft:loom": "shepherd",
    "minecraft:stonecutter_block": "mason",
    "minecraft:dispenser": "fletcher",
    "minecraft:cauldron": "leatherworker",
    "minecraft:smoker": "butcher"
};

world.afterEvents.playerInteractWithBlock.subscribe(({ player, block }) => {
    if (player.matches({ families: ["zv"] })) {
        const profession = workstations[block.typeId];

        if (profession) {
            const { x, y, z } = block.location;
            player.runCommandAsync(`execute if entity @s[scores={mana=30..},hasitem={item=zzk:pow, location=slot.weapon.mainhand}] run summon zzk:zv_table ${x} ${y} ${z} 0 0 ${profession}`);
            player.runCommandAsync(`execute if entity @s[scores={mana=30..},hasitem={item=zzk:pow, location=slot.weapon.mainhand}] run scoreboard players remove @s mana 30`);
        }
    }
});

const unmilkableMobs = [
    "minecraft:cow", "minecraft:goat", "minecraft:mooshroom",
    "minecraft:tadpole", "minecraft:axolotl",
    "minecraft:cod", "minecraft:salmon", "minecraft:pufferfish", "minecraft:tropical_fish",
    "minecraft:item", "minecraft:falling_block", "minecraft:fireball", "minecraft:ender_pearl",
    "minecraft:arrow", "minecraft:thrown_trident"
];

const milklore = {
    "zd": "zombie",
    "skeleton": "skeleton",
    "zombie": "zombie",
    "ghast": "creeper",
    "zc": "creeper",
    "zw": "warden",
    "magma_cube": "slime",
    "zsk": "skeleton",
    "za": "breeze",
    "zb": "blaze"
};


world.afterEvents.itemUse.subscribe(event => {
    const zUser = event.source;
    const zItem = event.itemStack;

    if (zItem?.typeId === "minecraft:bucket") {
        const raycastResult = zUser.getEntitiesFromViewDirection({ maxDistance: 5 });

        if (raycastResult.length > 0) {
            const targetEntity = raycastResult[0].entity;
            if (!targetEntity) return;

            if (!zUser.matches({ families: ["zcw"] }) && !targetEntity.matches({ families: ["zcw"] })) return;
            if (targetEntity.matches({ families: ["inanimate"] }) || unmilkableMobs.includes(targetEntity.typeId)) return;

            let category = targetEntity.typeId.replace("minecraft:", ""); 

            for (const [family, name] of Object.entries(milklore)) {
                if (targetEntity.matches({ families: [family] })) {
                    category = name;
                    break;
                }
            }

            const mobName = targetEntity.nameTag || category;
            const milkBucket = new ItemStack("minecraft:milk_bucket", 1);
            milkBucket.nameTag = `§r§f${mobName}'s Milk`;
            milkBucket.setLore([`milked from : ${category}`]);

            const inventory = zUser.getComponent("minecraft:inventory");
            if (inventory) {
                inventory.container.addItem(milkBucket);
                zUser.runCommandAsync(`clear @s minecraft:bucket 0 1`);
            }
        }
    }
});



world.afterEvents.itemCompleteUse.subscribe(event => {
    const zUser = event.source;
    const zItem = event.itemStack;

    if (zItem?.typeId === "minecraft:milk_bucket") {
        const lore = zItem.getLore();


        if (lore.length > 0 && lore[0].startsWith("milked from : ")) {
            const entityType = lore[0].split(":")[1].trim();

            switch (entityType) {
                case "creeper":
                    zUser.runCommandAsync("summon zzk:explodiv ~~~~~ milkcreeper");
                    break;
                case "zombie":
                    zUser.runCommandAsync("effect @s poison 10 1");
                    break;
                case "warden":
                    zUser.runCommandAsync("effect @s blindness 10 1");
                    break;
                case "horse":
                    zUser.runCommandAsync("playsound mob.horse.death");
                    zUser.runCommandAsync("effect @s speed 20 2");
                    break;
                case "chicken":
                    zUser.runCommandAsync("playsound mob.chicken.hurt");
                    zUser.runCommandAsync("give @s egg 1");
                    break;
                case "pig":
                    zUser.runCommandAsync("playsound mob.pig.death");
                    break;
                case "sheep":
                    zUser.runCommandAsync("playsound mob.sheep.say");
                    zUser.runCommandAsync("give @s wool 1");
                    break;
                case "breeze":
                    zUser.runCommandAsync("summon wind_charge_projectile");
                    zUser.runCommandAsync("summon wind_charge_projectile");
                    break;
                case "slime":
                    zUser.runCommandAsync("effect @s jump_boost 20 2 true");
                    break;
                case "skeleton":
                    zUser.runCommandAsync("effect @s weakness 30 10");
                    zUser.runCommandAsync("playanimation @s animation.skinny x 10");
                    break;
                case "blaze":
                    if (Math.random() < 0.5) {
                        zUser.runCommandAsync("effect @s fire_resistance 30 10");
                    } else {
                        zUser.runCommandAsync("summon zzk:setfire");
                    }
                    break;

                default:
                    zUser.runCommandAsync("effect @s regeneration 1 5 true");
                    break;
            }
        }
    }
});


world.afterEvents.entityHurt.subscribe((event) => {
    const entity = event.hurtEntity;
    const damageSource = event.damageSource;

    if (entity.matches({ families: ["hero_balance"] }) && damageSource.cause === "lightning") {
        entity.runCommandAsync("execute if entity @e[family=lightning,r=5] run event entity @s lightningstruck");
    }
});


const messages = [
    "Even pillagers spare the babies...",
    "You monster! That was just a child!",
    "The Iron Golems will remember this.",
    "Villagers whisper your name in fear now.",
    "Not even the pillagers would stoop this low.",
    "Even the Illagers would shake their heads at this.",
    "Professionals have standards."
];

world.afterEvents.entityDie.subscribe((event) => {
    const { deadEntity, damageSource } = event;
    let killer = damageSource.damagingEntity;

    if (killer && deadEntity.matches({ families: ["villager"] }) && killer.matches({ families: ["zp"] })) {
        let isBaby = deadEntity.getComponent("is_baby");

        if (isBaby) {
            let randomMessage = messages[Math.floor(Math.random() * messages.length)];
            killer.sendMessage("§4"+randomMessage);
            killer.runCommand("scoreboard players remove @s[scores={cd1=20..}] cd1 20");
            killer.runCommand("scoreboard players remove @s[scores={mana=0..}] mana 50");
        } else {
            killer.runCommand("scoreboard players add @s cd1 1");
            killer.runCommand("scoreboard players add @s[scores={mana=..99}] mana 30");
            killer.runCommand('tellraw @s {"rawtext":[{"text":"§4Villager kill count : "},{"score":{"name":"@s","objective":"cd1"}}]}');
        }
    }
});




system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const effects = player.getEffects();

    for (const effect of effects) {
      if (effect.typeId === "bad_omen") {
        const villagersNearby = player.dimension.getEntities({
          location: player.location,
          maxDistance: 20,
          type: "minecraft:villager"
        });

        if (villagersNearby.length > 0) {
          player.runCommandAsync("effect @s clear bad_omen");
          player.runCommandAsync("effect @s raid_omen");
          player.runCommandAsync("event entity @s minecraft:remove_raid_trigger");
        }
      }
    }
  }
}, 100); // every 5 seconds

system.runInterval(() => {
  for (const player of world.getAllPlayers()) {
    const effects = player.getEffects();

    for (const effect of effects) {
      if (effect.typeId === "raid_omen") {
        player.runCommandAsync("event entity @s minecraft:trigger_raid");
        break;
      }
    }
  }
}, 60); // every 3 seconds